/*
 *  This file is part of uIRC. (https://git.redxen.eu/caskd/uIRC)
 *  Copyright (c) 2019, 2020 Alex-David Denes
 *
 *  uIRC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  uIRC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with uIRC.  If not, see <https://www.gnu.org/licenses/>.
 */
#include "validators.h"

signed int
Val_mesg(IRC_Message* mesg)
{
	if (mesg == NULL) return ERR_UIRC_NULL_ARGS;
	for (unsigned int i = 0; mesg->args[i] != NULL; i++) {
		if (Val_type_nocrlf(mesg->args[i]) != 1) return ERR_UIRC_VAL_FAILED;
		if (!(mesg->args[i + 1] == NULL && mesg->trailing)) {
			if (Val_type_nospcl(mesg->args[i]) != 1) return ERR_UIRC_VAL_FAILED;
		}
	}
	return 1;
}

signed int
Val_type_nocrlf(char* str)
{
	if (str == NULL) return ERR_UIRC_NULL_ARGS;
	for (; *str; str++) {
		if (*str == '\r' || *str == '\n') return 0;
	}
	return 1;
}

signed int
Val_type_nospcl(char* str)
{
	if (str == NULL) return ERR_UIRC_NULL_ARGS;
	for (; *str; str++) {
		if (*str == ' ' || *str == ':') return 0;
	}
	return 1;
}

signed int
Val_type_noblcm(char* str)
{
	if (str == NULL) return ERR_UIRC_NULL_ARGS;
	for (; *str; str++) {
		if (*str == '\a' || *str == ',') return 0;
	}
	return 1;
}

signed int
Val_channame(char* chan)
{
	if (chan == NULL) return ERR_UIRC_NULL_ARGS;
	if (*chan != '#' && *chan != '+' && *chan != '!' && *chan != '&') return 0;
	char* clps = ++chan;
	if ((clps = strchr(chan, ':')) != NULL) {
		*clps = '\0';
		if (Val_type_nospcl(chan) != 1 || Val_type_nocrlf(chan) != 1 || Val_type_noblcm(chan) != 1) {
			*clps = ':';
			return 0;
		}
		*clps = ':';
		chan  = ++clps;
	}
	if (Val_type_nospcl(chan) != 1 || Val_type_nocrlf(chan) != 1 || Val_type_noblcm(chan) != 1) return 0;
	return 1;
}

